
/*
  Copyright 1995 The University of Rhode Island and The Massachusetts
  Institute of Technology

  Portions of this software were developed by the Graduate School of
  Oceanography (GSO) at the University of Rhode Island (URI) in collaboration
  with The Massachusetts Institute of Technology (MIT).

  Access and use of this software shall impose the following obligations and
  understandings on the user. The user is granted the right, without any fee
  or cost, to use, copy, modify, alter, enhance and distribute this software,
  and any derivative works thereof, and its supporting documentation for any
  purpose whatsoever, provided that this entire notice appears in all copies
  of the software, derivative works and supporting documentation.  Further,
  the user agrees to credit URI/MIT in any publications that result from the
  use of this software or in any product that includes this software. The
  names URI, MIT and/or GSO, however, may not be used in any advertising or
  publicity to endorse or promote any products or commercial entity unless
  specific written permission is obtained from URI/MIT. The user also
  understands that URI/MIT is not obligated to provide the user with any
  support, consulting, training or assistance of any kind with regard to the
  use, operation and performance of this software nor to provide the user
  with any updates, revisions, new versions or "bug fixes".

  THIS SOFTWARE IS PROVIDED BY URI/MIT "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
  EVENT SHALL URI/MIT BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
  DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
  PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTUOUS
  ACTION, ARISING OUT OF OR IN CONNECTION WITH THE ACCESS, USE OR PERFORMANCE
  OF THIS SOFTWARE.
*/

// Implementation for JGByte. See the comments in JGByte.h.  For each of the
// `variable classes' (e.g., Byte, ... Array, ... Grid) you *must* define a
// ctor, dtor, ptr_duplicate and read mfunc. In addition, you must edit the
// definition of New<class name> so that it creates the correct type of
// object. for example, edit NewByte() so that it creates and returns a
// JGByte pointer (see util.cc).
//
// jhrg 1/12/95

// $Log: JGByte.cc,v $
// Revision 1.8  1999/05/05 04:38:00  brent
// convert String.h code to new stanard lib <string>
//
// Revision 1.7  1998/08/06 16:32:33  jimg
// Fixed misuse of the read(...) member function. Return true if more data
// is to be read, false is if not and error if an error is detected
//
// Revision 1.6  1996/11/12 16:55:11  jimg
// Added config_jg.h include
//
// Revision 1.5  1996/09/17 00:01:53  jimg
// Misc changes associated with merging branch code to the trunk.
//
// Revision 1.4.6.1.2.1  1996/06/26 22:17:42  jimg
// Dans changes for 2.05.
//
// Revision 1.4  1995/10/11  23:06:05  jimg
// These files are the result of merging Dan's changes into the main
// trunk. However, the archive was messed up in the process, so it may be hard
// to trace back to the original versions of some of these files.
//
// Revision 1.3  1995/07/09  21:39:44  jimg
// Added copyright notice.
//
// Revision 1.2  1995/03/14  01:23:22  jimg
// Modified so that these files are a proper subclass of the DAP classes
// Byte, ..., Grid.
// Added assert macros.
// Added Jgofs.h external declarations file.
//
// Revision 1.1  1995/02/16  18:59:01  dan
// JGOFS sub-class definitions for Byte.cc
//
// Revision 1.2  1995/01/19  21:58:51  jimg
// Added read_val from dummy_read.cc to the sample set of sub-class
// implementations.
// Changed the declaration of readVal in BaseType so that it names the
// mfunc read_val (to be consistant with the other mfunc names).
// Removed the unnecessary duplicate declaration of the abstract virtual
// mfuncs read and (now) read_val from the classes Byte, ... Grid. The
// declaration in BaseType is sufficient along with the decl and definition
// in the *.cc,h files which contain the subclasses for Byte, ..., Grid.
//
// Revision 1.1  1995/01/19  20:20:37  jimg
// Created as an example of subclassing the class hierarchy rooted at
// BaseType.
//

#include "config_jg.h"

#ifdef __GNUG__
#pragma implementation
#endif

#include <assert.h>

#include "JGByte.h"
#include "jgofs.h"

// The NewByte `helper function' creates a pointer to the a JGByte and
// returns that pointer. It takes the same arguments as the class's ctor. If
// any of the variable classes are subclassed (e.g., to make a new Byte like
// HDFByte) then the corresponding function here, and in the other class
// definition files, needs to be changed so that it creates an instnace of
// the new (sub)class. Continuing the earlier example, that would mean that
// NewByte() would return a HDFByte, not a Byte.
//
// It is important that these function's names and return types do not change
// - they are called by the parser code (for the dds, at least) so if their
// names changes, that will break.
//
// The declarations for these fuctions (in util.h) should *not* need
// changing. 

Byte *
NewByte(const string &n)
{
    return new JGByte(n);
}

JGByte::JGByte(const string &n) : Byte(n)
{
}

BaseType *
JGByte::ptr_duplicate()
{
    return new JGByte(*this);
}

bool
JGByte::read(const string &dataset, int &error)
{
    error = 1;
    return false;
}




