
// -*- C++ -*-

// JGByte is a demonstration of subclassing in the hierarchy of DODS data
// types. It does not do much of anything - a real subclass would add
// specifics for a API or format (e.g., a read mfunc for netcdf, HDF, ...).
// The class is used by some of the test code (hence the name) as well as
// serving as a template for others who need to subclass the hierarchy.
//
// NB: since the class Byte is an abstract class (as are all the other
// `variable type' classes), the hierarchy *must* be subclassed in order to
// be used.
//
// jhrg 1/12/95

/* $Log: JGByte.h,v $
/* Revision 1.4  1999/05/05 04:38:00  brent
/* convert String.h code to new stanard lib <string>
/*
/* Revision 1.3  1996/09/17 00:02:07  jimg
/* Misc changes associated with merging branch code to the trunk.
/*
 * Revision 1.2.8.1.2.1  1996/06/26 22:17:43  jimg
 * Dans changes for 2.05.
 *
 * Revision 1.2  1995/03/14  01:23:34  jimg
 * Modified so that these files are a proper subclass of the DAP classes
 * Byte, ..., Grid.
 * Added assert macros.
 * Added Jgofs.h external declarations file.
 *
 * Revision 1.1  1995/02/16  18:59:20  dan
 * JGOFS sub-class definitions for Byte.h
 *
 * Revision 1.2  1995/01/19  21:59:30  jimg
 * Added read_val from dummy_read.cc to the sample set of sub-class
 * implementations.
 * Changed the declaration of readVal in BaseType so that it names the
 * mfunc read_val (to be consistant with the other mfunc names).
 * Removed the unnecessary duplicate declaration of the abstract virtual
 * mfuncs read and (now) read_val from the classes Byte, ... Grid. The
 * declaration in BaseType is sufficient along with the decl and definition
 * in the *.cc,h files which contain the subclasses for Byte, ..., Grid.
 *
 * Revision 1.1  1995/01/19  20:20:38  jimg
 * Created as an example of subclassing the class hierarchy rooted at
 * BaseType.
 *
 */

#ifndef _JGByte_h
#define _JGByte_h 1

#ifdef __GNUG__
#pragma interface
#endif

#include "Byte.h"

class JGByte: public Byte {
public:
    JGByte(const string &n = (char *)0);
    virtual ~JGByte() {}

    virtual BaseType *ptr_duplicate();

    virtual bool read(const string &dataset, int &error);

};

typedef JGByte * JGBytePtr;

#endif

