
// -*- C++ -*-

// Interface for the class JGSequence.
//
// jhrg 1/12/95

// Note that JGOFS handles reading sequences using a function that returns
// all the values in the dataset (effectively flattening the a hierarchy of
// sequences) along with information about the level at which data values
// changed.
//
// JGOFS assumes a certain layout is present in all datasets. Every dataset
// consists of a sequence which contains the entire dataset. Within that
// sequence there may be simple types (Floats and Strings only) and one (I
// think its limited to one) other Sequence. Each sequence must follow these
// rules - it must contain 1 or more simple-type memebers and may contain at
// most one subsequence.
//
// jhrg 1/6/98

/* 
 * $Log: JGSequence.h,v $
 * Revision 1.8  1999/05/05 04:38:00  brent
 * convert String.h code to new stanard lib <string>
 *
 * Revision 1.7  1998/08/06 16:32:35  jimg
 * Fixed misuse of the read(...) member function. Return true if more data
 * is to be read, false is if not and error if an error is detected
 *
 * Revision 1.6  1998/02/19 19:48:19  jimg
 * Reverted to t eh old read() member function.
 * Removed stuff for the optimized Sequence transfer.
 *
 * Revision 1.5  1998/02/05 20:14:34  jimg
 * DODS now compiles with gcc 2.8.x
 *
 * Revision 1.4  1997/12/16 00:50:08  jimg
 * Fixed errant Log comments.
 *
 * Revision 1.3  1996/09/17 00:02:17  jimg
 * Misc changes associated with merging branch code to the trunk.
 *
 * Revision 1.2.8.1.2.1  1996/06/26 22:18:03  jimg
 * Dans changes for 2.05.
 *
 * Revision 1.2  1995/03/14  01:23:40  jimg
 * Modified so that these files are a proper subclass of the DAP classes
 * Byte, ..., Grid.
 * Added assert macros.
 * Added Jgofs.h external declarations file.
 *
 * Revision 1.1  1995/02/16  19:13:39  dan
 * JGOFS sub-class definitions for Sequence.h
 *
 * Revision 1.2  1995/01/19  21:59:42  jimg
 * Added read_val from dummy_read.cc to the sample set of sub-class
 * implementations.
 * Changed the declaration of readVal in BaseType so that it names the
 * mfunc read_val (to be consistent with the other mfunc names).
 * Removed the unnecessary duplicate declaration of the abstract virtual
 * mfuncs read and (now) read_val from the classes Byte, ... Grid. The
 * declaration in BaseType is sufficient along with the decl and definition
 * in the *.cc,h files which contain the subclasses for Byte, ..., Grid.
 *
 * Revision 1.1  1995/01/19  20:20:52  jimg
 * Created as an example of subclassing the class hierarchy rooted at
 * BaseType.
 */

#ifndef _JGSequence_h
#define _JGSequence_h 1

#ifdef _GNUG_
#pragma interface
#endif

#include "Sequence.h"
#include "DDS.h"

class JGSequence: public Sequence {
private:
    virtual bool read_level(const string &dataset, JGSequence*, int &error);

#if 0
    static int _current_level;
    int _use_buffer;
#endif

public:
    JGSequence(const string &n = (char *)0);
    virtual ~JGSequence();

    virtual BaseType *ptr_duplicate();

    virtual bool read(const string &dataset, int &error);

#if 0
    /** Return true if the JGSequence object cotains other Sequence objects,
        false otherwise. */
    bool JGSequence::child_sequences_p();

    /** Read the value of the current level. Although this value is set
        within one of potentially several recursive calls to
	JGSequence::read(). This mfunc accesses a static member so all
	objects can read the same value. */ 
    int current_level();

    /** Set the current level. All instances of the class JGSequence can
        access this value. Thus a recursive call can update the current level
	and, by setting the current level with this mfunc, pass that
	information to its caller and on up the call chain. */
    void set_current_level(int cl);

    /** Is valid data in the buffer filled by jdbread()? Returns true if data
        is in the buffer, false otherwise. */
    bool use_buffer_p();

    /** Set the value returned by use_buffer_p to state. */
    void set_use_buffer_p(bool stte);
#endif
};

typedef JGSequence * JGSequencePtr;

#endif


