
// This is the main CGI program which extracts DAS class attributes from a
// netCDF data file. In addition, the attributes from an external file (if 
// exist, with the same name as the netCDF file and extension of .das at the 
// end) are extracted and added to the in-memory atrribute table before it is 
// sent over the network to the client side. 
//
// ReZa 9/29/94
// DanH 10/26/94

// $Log: jg_das.cc,v $
// Revision 1.13  1999/05/25 21:52:07  dan
// Added test to determine location of ancillary DAS files, relative
// to the location of the primary datafile defined in the JGOFS object
// dictionary for the data object called by the URL.  This now works
// with the new modifications to DODSFilter::read_ancillary_das().
//
// Revision 1.12  1999/05/25 19:13:28  dan
// Migrated to DODSFilter usage for command line processing.
//
// Revision 1.11  1999/05/05 22:57:19  jimg
// Merged Brent's String --> string changes with Dan's changes
//
// Revision 1.10  1999/03/17 21:42:20  dan
// Added changes to use JGOFS_OBJECT and JGOFS_METHOD variables
// to determine filesystem locations.
//
// Revision 1.9  1998/03/20 00:29:59  jimg
// Changed calls to the set_mime_*() to match the new parameters
//
// Revision 1.8  1997/07/01 14:11:16  dan
// Modified handling of ancillary DAS files to make them work correctly.
//
// Added references to <strstream.h> to rework the way I handle String
// objects.
//
// Revision 1.7  1997/03/10 22:56:50  dan
// Added Error-handling.
//
// Revision 1.6  1996/11/12 16:59:15  jimg
// Added MIME error returns for some errors.
//
// Revision 1.5  1996/09/17 00:04:09  jimg
// Misc changes associated with merging branch code to the trunk.
//
// Revision 1.4.6.1.2.1  1996/06/26 22:18:19  jimg
// Dans changes for 2.05.
//
// Revision 1.4  1995/10/11  23:06:24  jimg
// These files are the result of merging Dan's changes into the main
// trunk. However, the archive was messed up in the process, so it may be hard
// to trace back to the original versions of some of these files.
//
// Revision 1.3.2.1  1995/09/18  22:48:03  jimg
// Merged Dan's changes with the trunk version. This will be commited to the
// release-1-0-patches branch and then merged back into the trunk later.
//
// Revision 1.3  1995/07/09  21:40:00  jimg
// Added copyright notice.
//
// Revision 1.2  1995/03/14  02:15:11  jimg
// Modified for the new netio library.
// Removed old code which used C style strings (now it uses C++ Strings).
// Fixed misc bugs.
// Added comments.
//
// Revision 1.1  1995/01/09  16:04:46  dan
// These files constitute the JGOFS server and client library code
// for use with the DODS api.
//
// Revision 1.2  1994/10/06  17:53:06  jimg
// Some reformatting of code for emacs, ...
// Fixed Makefile.in so that two targets are built: client and server.
// Fixed cast away of const in ncdas.cc.
//

/*
   #define OBJECT_ROOT "/usr/local/jgofs/objects"
   #define METHOD_ROOT "/usr/local/jgofs/methods/"
*/

#define HOME_DIR_HACK 1

static char rcsid[]={"$Id: jg_das.cc,v 1.13 1999/05/25 21:52:07 dan Exp $"};

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <assert.h>

#include <iostream>
#include <strstream>
#include <string>

char *getenv();
#include <unistd.h>
#include <pwd.h>

#include "DAS.h"
#include "cgi_util.h"
#include "DODSFilter.h"

#include "jgofs.h"

extern bool create_das(DAS &das, string filename, string &error);
extern "C" int ldctsearch(const char*, char*, char*, int);

#include <sys/stat.h>

int Test_JgofsPath(char* pathname)
{
    struct stat buf;
    int i;

    if ( lstat(pathname,&buf) < 0 ) return 0;
    else {
      if ( S_ISDIR(buf.st_mode) || S_ISLNK(buf.st_mode)) return 1;
      return 0;
    }
}

int 
main(int argc, char * argv[])
{
    DAS das;
    DODSFilter df(argc, argv);

    if (!df.OK()) {
	df.print_usage();
	return 1;
    }

    if (df.version()) {
	df.send_version_info();
	return 0;
    }

/* The HOME_DIR_HACK uses the current machine's /etc/passwd file (or
   whatever getpwnam(3) searches) to find the JGOFS home directory. We
   assume that the jgofs object root directory is a subdir of the home
   dir called `objects'. This seems to be OK to do since the Makefile
   uses and equivalent assumption when it builds the JGOFS API. 10/13/95
   jhrg */

/* HOME_DIR_HACK modified to search for the environment variables
   `JGOFS_OBJECT', and `JGOFS_METHOD' to satisfy the JGOFS dictionary
   search mechanism without requiring an entry in the /etc/passwd file.
   3/1/99 danh */

    char *object_path, OBJECT_PATH[128];
    char *method_path, METHOD_PATH[128];

#if HOME_DIR_HACK

#include <sys/stat.h>

    struct passwd *pwd_entry;
    struct stat buf;
    bool object_error,method_error;
    char error[256];

    /* No `jgofs' user, OBJECT_PATH, METHOD_PATH must be defined, else exit with error */
    object_path = getenv("JGOFS_OBJECT");
    if ( object_path ) {
       object_error = FALSE;
       if (Test_JgofsPath(object_path) == 0) {
	 object_error = TRUE;
       }
       else {
	 method_path = getenv("JGOFS_METHOD");
	 if ( method_path ) {
	   method_error = FALSE;
	   if (Test_JgofsPath(method_path) == 0) {
	     method_error = TRUE;
	   }
	 }
	 else {  /* METHOD_PATH is undefined */
	   method_error = TRUE;
	 }
       }
    }
    else {
      pwd_entry = getpwnam( "jgofs" );	/* assume the name "jgofs" */
      if ( pwd_entry == NULL ) {
	/* OBJECT_PATH is undefined && No `jgofs' user defined. */
	object_error = TRUE;
      }
      else {  
	
	/* `jgofs' user exists, define OBJECT_PATH, METHOD_PATH relative to home directory */
	object_path = OBJECT_PATH;
	strcpy(object_path, pwd_entry->pw_dir);		
	strcat(object_path, "/objects");        /* assume objects are in subdir `objects' */
	object_error = FALSE;
	if (Test_JgofsPath(object_path) == 0) object_error = TRUE;
	
	method_path = METHOD_PATH;
	strcpy(method_path, pwd_entry->pw_dir);
	strcat(method_path, "/methods/");	/* assume methods are in subdir `methods' */
	method_error = FALSE;
	if (Test_JgofsPath(method_path) == 0) method_error = TRUE;
      }
    }
    /*
     * Check for errors in the OBJECT_PATH and METHOD_PATH locations.
     */
    if ( object_error || method_error ) {
      sprintf(error, "JGOFS_OBJECT, JGOFS_METHOD, or user 'jgofs' undefined, refer to INSTALL-jgofs");
      Error *ErrorObj = new Error(no_such_file, (string)error);
      set_mime_text(cout, dods_error);
      ErrorObj->print(cout);
      exit(1);
    }

#else
    /*
     * HOME_DIR_HACK is not set
     */
    object_path = OBJECT_PATH;
    strcpy(object_path, OBJECT_ROOT);	/* original objects */
      object_error = FALSE;
      if (Test_JgofsPath(object_path) == 0) object_error = TRUE;

    method_path = METHOD_PATH;
    strcpy(method_path, METHOD_ROOT);	/* original method  */
      method_error = FALSE;
      if (Test_JgofsPath(method_path) == 0) method_error = TRUE;

#endif

    // Read the JGOFS data-object attributes in memory and forward 
    // all the errors to client program.

    try {
	string errMsg;

	// if external attribute file exist, read it ...
	// This file must be located in the same directory that 
	// the JGOFS data-object is located, and named the
	// JGOFS data-object filename with suffix '.dds'.

	if ( !create_das(das, df.get_dataset_name(), errMsg) ) {      
	    Error e(no_such_file, errMsg);
	    set_mime_text(cout, dods_error, df.get_cgi_version());
	    e.print(cout);
	    return 1;
	}

	// Determine location of the initial file for the
	// JGOFS data object, using the JGOFS object dictionary.

	char met[256], par[256];
	string pathname = "";       

	if ( ldctsearch( df.get_dataset_name().data(),met,par,1) ) {
	  string objname = par;
	  size_t slash = objname.find_last_of('/');
	  pathname = objname.substr(0, slash+1);
	}

	if (!df.read_ancillary_das(das,pathname))
	    return 1;

	if (!df.send_das(das))
	    return 1;
    }
    catch (Error &e) {
	set_mime_text(cout, dods_error, df.get_cgi_version());
	e.print(cout);
	return 1;
    }

    return 0;
}


















