
/*
  Copyright 1995 The University of Rhode Island and The Massachusetts
  Institute of Technology

  Portions of this software were developed by the Graduate School of
  Oceanography (GSO) at the University of Rhode Island (URI) in collaboration
  with The Massachusetts Institute of Technology (MIT).

  Access and use of this software shall impose the following obligations and
  understandings on the user. The user is granted the right, without any fee
  or cost, to use, copy, modify, alter, enhance and distribute this software,
  and any derivative works thereof, and its supporting documentation for any
  purpose whatsoever, provided that this entire notice appears in all copies
  of the software, derivative works and supporting documentation.  Further,
  the user agrees to credit URI/MIT in any publications that result from the
  use of this software or in any product that includes this software. The
  names URI, MIT and/or GSO, however, may not be used in any advertising or
  publicity to endorse or promote any products or commercial entity unless
  specific written permission is obtained from URI/MIT. The user also
  understands that URI/MIT is not obligated to provide the user with any
  support, consulting, training or assistance of any kind with regard to the
  use, operation and performance of this software nor to provide the user
  with any updates, revisions, new versions or "bug fixes".

  THIS SOFTWARE IS PROVIDED BY URI/MIT "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
  EVENT SHALL URI/MIT BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
  DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
  PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTUOUS
  ACTION, ARISING OUT OF OR IN CONNECTION WITH THE ACCESS, USE OR PERFORMANCE
  OF THIS SOFTWARE.
*/

// This is the main CGI program which extracts DDS class attributes from a
// JGOFS data file. In addition, the attributes from an external file (if 
// exist, with the same name as the JGOFS data-object file with suffix '.dds')
// are extracted and added to the in-memory atrribute table before it is 
// sent over the network to the client side. 
//
// DanH 10/21/94

// $Log: jg_dds.cc,v $
// Revision 1.13  1999/05/27 15:50:38  dan
// Added new logic to find ancillary DDS files relative to the
// location of the top-most file defining a JGOFS data object,
// as defined in the object dictionary.
//
// Revision 1.12  1999/05/25 16:32:41  dan
// Migrated to DODSFilter for command-line processing.
//
// Revision 1.11  1999/05/05 04:38:00  brent
// convert String.h code to new stanard lib <string>
//
// Revision 1.10  1998/03/20 00:30:00  jimg
// Changed calls to the set_mime_*() to match the new parameters
//
// Revision 1.9  1997/06/06 03:54:13  jimg
// Changed call of create_dds() to the new version of that function.
//
// Revision 1.8  1997/03/27 18:29:30  jimg
// Update for version 2.13
//
// Revision 1.7  1997/03/10 23:01:37  dan
// Added DODS Error-handling.
//
// Revision 1.6  1996/11/12 17:01:15  jimg
// Added MIME error returns for certain errors.
//
// Revision 1.5  1996/09/17 00:04:10  jimg
// Misc changes associated with merging branch code to the trunk.
//
// Revision 1.4.6.1.2.1  1996/06/26 22:18:20  jimg
// Dans changes for 2.05.
//
// Revision 1.4  1995/10/11  23:06:26  jimg
// These files are the result of merging Dan's changes into the main
// trunk. However, the archive was messed up in the process, so it may be hard
// to trace back to the original versions of some of these files.
//
// Revision 1.3.2.1  1995/09/18  22:48:04  jimg
// Merged Dan's changes with the trunk version. This will be commited to the
// release-1-0-patches branch and then merged back into the trunk later.
//
// Revision 1.3  1995/07/09  21:40:02  jimg
// Added copyright notice.
//
// Revision 1.2  1995/03/14  02:15:13  jimg
// Modified for the new netio library.
// Removed old code which used C style strings (now it uses C++ Strings).
// Fixed misc bugs.
// Added comments.
//
// Revision 1.1  1995/01/09  16:04:47  dan
// These files constitute the JGOFS server and client library code
// for use with the DODS api.
//
// Revision 1.2  1994/10/06  17:53:06  jimg
// Some reformatting of code for emacs, ...
// Fixed Makefile.in so that two targets are built: client and server.
// Fixed cast away of const in ncdas.cc.

/*
   #define OBJECT_ROOT "/usr/local/jgofs/objects"
   #define METHOD_ROOT "/usr/local/jgofs/methods/"
*/

#define HOME_DIR_HACK 1

static char rcsid[]={"$Id: jg_dds.cc,v 1.13 1999/05/27 15:50:38 dan Exp $"};

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <assert.h>
#include <iostream.h>
#include <string>

#include "DDS.h"
#include "cgi_util.h"
#include "DODSFilter.h"

extern bool create_dds(DDS &dds, string jg_name, string &error);
extern "C" int ldctsearch(const char*, char*, char*, int);

#include <sys/stat.h>

int Test_JgofsPath(char* pathname)
{
    struct stat buf;
    int i;

    if ( lstat(pathname,&buf) < 0 ) return 0;
    else {
      if ( S_ISDIR(buf.st_mode) || S_ISLNK(buf.st_mode)) return 1;
      return 0;
    }
}

int 
main(int argc, char * argv[])
{
    DDS dds;
    DODSFilter df(argc, argv);
	
    if (!df.OK()) {
	df.print_usage();
	return 1;
    }

    if (df.version()) {
	df.send_version_info();
	return 0;
    }

/* The HOME_DIR_HACK uses the current machine's /etc/passwd file (or
   whatever getpwnam(3) searches) to find the JGOFS home directory. We
   assume that the jgofs object root directory is a subdir of the home
   dir called `objects'. This seems to be OK to do since the Makefile
   uses and equivalent assumption when it builds the JGOFS API. 10/13/95
   jhrg */

/* HOME_DIR_HACK modified to search for the environment variables
   `JGOFS_OBJECT', and `JGOFS_METHOD' to satisfy the JGOFS dictionary
   search mechanism without requiring an entry in the /etc/passwd file.
   3/1/99 danh */

    char *object_path, OBJECT_PATH[128];
    char *method_path, METHOD_PATH[128];

#if HOME_DIR_HACK

#include <sys/stat.h>

    struct passwd *pwd_entry;
    struct stat buf;
    bool object_error,method_error;
    char error[256];

    /* No `jgofs' user, OBJECT_PATH, METHOD_PATH must be defined, else exit with error */
    object_path = getenv("JGOFS_OBJECT");
    if ( object_path ) {
       object_error = FALSE;
       if (Test_JgofsPath(object_path) == 0) {
	 object_error = TRUE;
       }
       else {
	 method_path = getenv("JGOFS_METHOD");
	 if ( method_path ) {
	   method_error = FALSE;
	   if (Test_JgofsPath(method_path) == 0) {
	     method_error = TRUE;
	   }
	 }
	 else {  /* METHOD_PATH is undefined */
	   method_error = TRUE;
	 }
       }
    }
    else {
      pwd_entry = getpwnam( "jgofs" );	/* assume the name "jgofs" */
      if ( pwd_entry == NULL ) {
	/* OBJECT_PATH is undefined && No `jgofs' user defined. */
	object_error = TRUE;
      }
      else {  
	
	/* `jgofs' user exists, define OBJECT_PATH, METHOD_PATH relative to home directory */
	object_path = OBJECT_PATH;
	strcpy(object_path, pwd_entry->pw_dir);		
	strcat(object_path, "/objects");        /* assume objects are in subdir `objects' */
	object_error = FALSE;
	if (Test_JgofsPath(object_path) == 0) object_error = TRUE;
	
	method_path = METHOD_PATH;
	strcpy(method_path, pwd_entry->pw_dir);
	strcat(method_path, "/methods/");	/* assume methods are in subdir `methods' */
	method_error = FALSE;
	if (Test_JgofsPath(method_path) == 0) method_error = TRUE;
      }
    }
    /*
     * Check for errors in the OBJECT_PATH and METHOD_PATH locations.
     */
    if ( object_error || method_error ) {
      sprintf(error, "JGOFS_OBJECT, JGOFS_METHOD, or user 'jgofs' undefined, refer to INSTALL-jgofs");
      Error *ErrorObj = new Error(no_such_file, (string)error);
      set_mime_text(cout, dods_error);
      ErrorObj->print(cout);
      exit(1);
    }

#else
    /*
     * HOME_DIR_HACK is not set
     */
    object_path = OBJECT_PATH;
    strcpy(object_path, OBJECT_ROOT);	/* original objects */
      object_error = FALSE;
      if (Test_JgofsPath(object_path) == 0) object_error = TRUE;

    method_path = METHOD_PATH;
    strcpy(method_path, METHOD_ROOT);	/* original method  */
      method_error = FALSE;
      if (Test_JgofsPath(method_path) == 0) method_error = TRUE;

#endif

    // Set the filename. 12/22/98 jhrg
    dds.filename(df.get_dataset_name());

    // This software is between using return values for error reporting and
    // using exceptions. It should change to exceptions only. 5/3/99 jhrg
    try {
	// Open the JGOFS data-object and create a DataSet Descriptor Structure
	string errMsg;
	if (!create_dds(dds, df.get_dataset_name(), errMsg)) {
	    ErrMsgT("jg_dods: could not read the DDS information");
	    //client error message
	    Error e(unknown_error, errMsg);
	    set_mime_text(cout, dods_error);
	    e.print(cout);
	    return 1;
	}

	// Determine location of the initial file for the
	// JGOFS data object, using the JGOFS object dictionary.

	char met[256], par[256];
	string pathname = "";       

	if ( ldctsearch( df.get_dataset_name().data(),met,par,1) ) {
	  string objname = par;
	  size_t slash = objname.find_last_of('/');
	  pathname = objname.substr(0, slash+1);
	}

	if (!df.read_ancillary_dds(dds,pathname))
	    return 1;

	if (!df.send_dds(dds, stdout))
	    return 1;
    }
    catch (Error &e) {
	e.print(cout);
	return 1;
    }

    return 0;
}







