
// This is the main CGI program which serves JGOFS data using the DODS DAP. 
//
// DanH 12/20/94

// $Log: jg_dods.cc,v $
// Revision 1.17  1999/05/27 15:51:32  dan
// Added logic to find ancillary DDS file relative to the
// top-most file defining a JGOFS data object, as specified
// in the object dictionary.
//
// Revision 1.16  1999/05/25 16:33:55  dan
// Removed extraneous extern definitions.
//
// Revision 1.15  1999/05/05 22:57:19  jimg
// Merged Brent's String --> string changes with Dan's changes
//
// Revision 1.14  1999/01/21 01:46:14  jimg
// Added call to set filename in the DDS. This is required for proper operation
// of the projection functions.
//
// Revision 1.13  1998/03/20 00:30:00  jimg
// Changed calls to the set_mime_*() to match the new parameters
//
// Revision 1.12  1997/08/05 22:45:01  jimg
// Changed type of option_char in `option_char = getopt(...)' from char to
// int. This fixes a bug that shows up on the SGI (where char is unsigned).
//
// Revision 1.11  1997/06/05 17:20:13  jimg
// Rewrote main() so that it uses functions added to cgi_util.cc for the 2.14
// release. These function help to unify the *_dods filters so that they all
// respond more uniformly.
//
// Revision 1.10  1997/03/13 06:12:26  jimg
// Fixed call to create_dds(). Now create_dds() returns an error message if an
// error was detected.
//
// Revision 1.9  1997/03/10 23:05:48  dan
// Modified code to reflect new sub-process changes used when
// compressing the data stream.
//
// Revision 1.7  1996/11/21 23:55:58  jimg
// Changed compression to new compressor function.
//
// Revision 1.6  1996/11/12 17:02:03  jimg
// Removed dead wood.
// Added MIME error returns for certain errors.
//
// Revision 1.5  1996/09/17 00:04:11  jimg
// Misc changes associated with merging branch code to the trunk.
//
// Revision 1.4.2.1.2.1  1996/06/26 22:18:21  jimg
// Dans changes for 2.05.
//
// Revision 1.2  1995/10/11  23:06:27  jimg
// These files are the result of merging Dan's changes into the main
// trunk. However, the archive was messed up in the process, so it may be hard
// to trace back to the original versions of some of these files.
//
// Revision 1.1  1995/09/29  23:31:56  jimg
// Added after much confusion and CVS screw-ups.
//
// Revision 1.1  1995/09/18  22:50:00  jimg
// These files changed as a result of edit Dan made to the jg-dods code so that
// the servers use exiting jgofs methods (i.e., one set of jg-dods servers can
// be used to access N methods, without the linking hack previously used).
//
// Revision 1.3  1995/03/14  02:15:14  jimg
// Modified for the new netio library.
// Removed old code which used C style strings (now it uses C++ Strings).
// Fixed misc bugs.
// Added comments.
//
// Revision 1.2  1995/01/31  21:31:14  dan
// Removed files from repository which have been moved to ../netio
//

/*
   #define OBJECT_ROOT "/usr/local/jgofs/objects"
   #define METHOD_ROOT "/usr/local/jgofs/methods/"
*/

#define HOME_DIR_HACK 1

static char rcsid[]={"$Id: jg_dods.cc,v 1.17 1999/05/27 15:51:32 dan Exp $"};

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <assert.h>

#include <iostream>
#include <string>

#include <GetOpt.h>

#include "DDS.h"
#include "Sequence.h"
#include "Str.h"
#include "cgi_util.h"
#include "DODSFilter.h"

#include "jgofs.h"
#include "JGSequence.h"
#include "JGStr.h"

extern bool create_dds(DDS &dds, string jg_name, string &error);

extern "C" {
    int ljdbreada_(int *, char (*)[VALSIZE], int *);
    int ljdbclose_(int *);
    int ldctsearch(const char*, char*, char*, int);
};

#include <sys/stat.h>

int Test_JgofsPath(char* pathname)
{
    struct stat buf;
    int i;

    if ( lstat(pathname,&buf) < 0 ) return 0;
    else {
      if ( S_ISDIR(buf.st_mode) || S_ISLNK(buf.st_mode)) return 1;
      return 0;
    }
}

int 
main(int argc, char * argv[])
{
    DDS dds;
    DODSFilter df(argc, argv);
	
    if (!df.OK()) {
	df.print_usage();
	return 1;
    }

    if (df.version()) {
	df.send_version_info();
	return 0;
    }

/* The HOME_DIR_HACK uses the current machine's /etc/passwd file (or
   whatever getpwnam(3) searches) to find the JGOFS home directory. We
   assume that the jgofs object root directory is a subdir of the home
   dir called `objects'. This seems to be OK to do since the Makefile
   uses and equivalent assumption when it builds the JGOFS API. 10/13/95
   jhrg */

/* HOME_DIR_HACK modified to search for the environment variables
   `JGOFS_OBJECT', and `JGOFS_METHOD' to satisfy the JGOFS dictionary
   search mechanism without requiring an entry in the /etc/passwd file.
   3/1/99 danh */

    char *object_path, OBJECT_PATH[128];
    char *method_path, METHOD_PATH[128];

#if HOME_DIR_HACK

#include <sys/stat.h>

    struct passwd *pwd_entry;
    struct stat buf;
    bool object_error,method_error;
    char error[256];

    /* No `jgofs' user, OBJECT_PATH, METHOD_PATH must be defined, else exit with error */
    object_path = getenv("JGOFS_OBJECT");
    if ( object_path ) {
       object_error = FALSE;
       if (Test_JgofsPath(object_path) == 0) {
	 object_error = TRUE;
       }
       else {
	 method_path = getenv("JGOFS_METHOD");
	 if ( method_path ) {
	   method_error = FALSE;
	   if (Test_JgofsPath(method_path) == 0) {
	     method_error = TRUE;
	   }
	 }
	 else {  /* METHOD_PATH is undefined */
	   method_error = TRUE;
	 }
       }
    }
    else {
      pwd_entry = getpwnam( "jgofs" );	/* assume the name "jgofs" */
      if ( pwd_entry == NULL ) {
	/* OBJECT_PATH is undefined && No `jgofs' user defined. */
	object_error = TRUE;
      }
      else {  
	
	/* `jgofs' user exists, define OBJECT_PATH, METHOD_PATH relative to home directory */
	object_path = OBJECT_PATH;
	strcpy(object_path, pwd_entry->pw_dir);		
	strcat(object_path, "/objects");        /* assume objects are in subdir `objects' */
	object_error = FALSE;
	if (Test_JgofsPath(object_path) == 0) object_error = TRUE;
	
	method_path = METHOD_PATH;
	strcpy(method_path, pwd_entry->pw_dir);
	strcat(method_path, "/methods/");	/* assume methods are in subdir `methods' */
	method_error = FALSE;
	if (Test_JgofsPath(method_path) == 0) method_error = TRUE;
      }
    }
    /*
     * Check for errors in the OBJECT_PATH and METHOD_PATH locations.
     */
    if ( object_error || method_error ) {
      sprintf(error, "JGOFS_OBJECT, JGOFS_METHOD, or user 'jgofs' undefined, refer to INSTALL-jgofs");
      Error *ErrorObj = new Error(no_such_file, (string)error);
      set_mime_text(cout, dods_error);
      ErrorObj->print(cout);
      exit(1);
    }

#else
    /*
     * HOME_DIR_HACK is not set
     */
    object_path = OBJECT_PATH;
    strcpy(object_path, OBJECT_ROOT);	/* original objects */
      object_error = FALSE;
      if (Test_JgofsPath(object_path) == 0) object_error = TRUE;

    method_path = METHOD_PATH;
    strcpy(method_path, METHOD_ROOT);	/* original method  */
      method_error = FALSE;
      if (Test_JgofsPath(method_path) == 0) method_error = TRUE;

#endif

    // Set the filename. 12/22/98 jhrg
    dds.filename(df.get_dataset_name());

    // This software is between using return values for error reporting and
    // using exceptions. It should change to exceptions only. 5/3/99 jhrg
    try {
	// Open the JGOFS data-object and create a DataSet Descriptor Structure
	string errMsg;
	if (!create_dds(dds, df.get_dataset_name(), errMsg)) {
	    ErrMsgT("jg_dods: could not read the DDS information");
	    //client error message
	    Error e(unknown_error, errMsg);
	    set_mime_text(cout, dods_error);
	    e.print(cout);
	    return 1;
	}

	// Determine location of the initial file for the
	// JGOFS data object, using the JGOFS object dictionary.

	char met[256], par[256];
	string pathname = "";       

	if ( ldctsearch( df.get_dataset_name().data(),met,par,1) ) {
	  string objname = par;
	  size_t slash = objname.find_last_of('/');
	  pathname = objname.substr(0, slash+1);
	}

	if (!df.read_ancillary_dds(dds,pathname))
	    return 1;

	if (!df.send_data(dds, stdout))
	    return 1;
    }
    catch (Error &e) {
	e.print(cout);
	return 1;
    }

    return 0;
}

