
// This file contains functions which read the variables and level
// information from a JGOFS data-object and build the in-memory DDS. These
// functions form the core of the server-side software necessary to extract
// the DDS from a JGOFS data-object file.
//
// It also contains test code which will print the in-memory DDS to
// stdout. It uses both the DDS class as well as the JGOFS object-method
// library. 
//
// jhrg 9/23/94 / dholloway 10/25/94

// $Log: jgdds.cc,v $
// Revision 1.17  1999/05/05 22:57:20  jimg
// Merged Brent's String --> string changes with Dan's changes
//
// Revision 1.16  1999/05/05 04:38:00  brent
// convert String.h code to new stanard lib <string>
//
// Revision 1.15  1998/08/06 16:32:41  jimg
// Fixed misuse of the read(...) member function. Return true if more data
// is to be read, false is if not and error if an error is detected
//
// Revision 1.14  1998/02/19 19:49:17  jimg
// Removed call to Sequence::set_level() since the Sequence::add_var() member
// function now does this.
//
// Revision 1.13  1998/02/05 20:14:39  jimg
// DODS now compiles with gcc 2.8.x
//
// Revision 1.12  1997/08/06 16:53:44  jimg
// Dan fixed nasty string bug where char * was used where String was
// expected.
//
// Revision 1.11  1997/06/05 17:19:27  jimg
// Removed the constraint parameter from create_dds() since it was not used.
// Changed type of parameter `filename' to String. Removed unused code.
//
// Revision 1.10  1997/03/10 23:02:51  dan
// Added Error-handling.  Add calls to new API modules which provide
// access to errors generated by local JGOFS methods.
//
// Revision 1.9  1996/11/09 00:43:18  jimg
// Increased the number of variable that can be processed from 99 to NVALS
// (250).
//
// Revision 1.8  1996/09/17 00:04:14  jimg
// Misc changes associated with merging branch code to the trunk.
//
// Revision 1.7.2.1.2.1  1996/06/26 22:18:25  jimg
// Dans changes for 2.05.
//
// Revision 1.5  1995/10/11  23:56:11  jimg
// Added external symbols defined/declared in jgofs.h to lgofs/ljdb.c. Fixed
// the C constant TOKEN so that it is `40' everywhere.
//
// Revision 1.4  1995/10/11  23:06:30  jimg
// These files are the result of merging Dan's changes into the main
// trunk. However, the archive was messed up in the process, so it may be hard
// to trace back to the original versions of some of these files.
//
// Revision 1.3.2.1  1995/09/18  22:48:07  jimg
// Merged Dan's changes with the trunk version. This will be commited to the
// release-1-0-patches branch and then merged back into the trunk later.
//
// Revision 1.3  1995/07/09  21:40:05  jimg
// Added copyright notice.
//
// Revision 1.2  1995/03/14  02:17:26  jimg
// Modified so that the new netio library is used.
// Modified so that C++ Strings are used in place of C strings.
// Removed old code copied from the dods netcdf sources.
//
// Revision 1.1  1995/01/09  16:04:53  dan
// These files constitute the JGOFS server and client library code
// for use with the DODS api.
//
// Revision 1.1  1994/10/25  9:00:00  dan
// First version of the jgofs software which used libdas++.a.
//

static char rcsid[]={"$Id: jgdds.cc,v 1.17 1999/05/05 22:57:20 jimg Exp $"};

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#include <iostream.h>
#include <strstream.h>
#include <string>

#include "DDS.h"
#include "cgi_util.h"

#include "jgofs.h"		// extern declarations including many globals.
#include "JGSequence.h"
#include "JGStr.h"

// Note that these globals are defined here and declared extern in `jgofs.h'.
// Including that file gives you access to these variables and is required
// when using the JGOFS code. jhrg 12/31/97.

char varName[NVALS][TOKEN];
char varValue[NVALS][VALSIZE];
int varLevel[NVALS];
int nVars, unit;
int varSize = TOKEN;

static JGSequence * 
CreateSequence(int currLvl, int maxLvls)
{
    if (currLvl > maxLvls) 
	return 0;		// stopping condition
    else {
	// Create a new 'Sequence' variable representation, then create a
	// 'BaseType' representation for each variable at this level and add
	// it to current Sequence. When done call CreateSequence with
	// currLvl++.
	ostrstream oss;

	oss << "Level_" << currLvl << ends;
	string name = oss.str();
	oss.freeze(0);

	JGSequence *seq = new JGSequence(name);

#if 0
	// This is now done by the Sequence class. Doing it here is
	// redundent. 2/17/98 jhrg.
	seq->set_level(currLvl);
#endif

	for (int i = 0; i < nVars; i++) {
	    if (varLevel[i] == currLvl) {
		JGStr *str = new JGStr(varName[i]);
		seq->add_var(str);
	    }
	}

	JGSequence *nxt;
	if ((nxt = CreateSequence(currLvl+1, maxLvls))) 
	    seq->add_var(nxt);

	return seq;

    }
}

// Given a reference to an instance of class DDS and a filename that refers
// to a JGOFS data object, use the JGOFS data object's method to extract all
// the level and variable information describing the data object's
// hierarchical data representation. Add the variables and level information
// to the instance of DDS.
//
// Returns: false if an error accessing the JGOFS data-object was detected,
// true otherwise. 

#if 0
extern char *name_path(const char *);
#endif

bool
create_dds(DDS &dds, string jg_name, string &error)
{
    char jg_name_c[256];
    size_t pos;

    strncpy(jg_name_c, jg_name.data(), 255);
    jg_name_c[255] = '\0';	// for the truly paranoid...

    // Changed -99 to -NVALS (250). jhrg 11/8/96
    nVars = -(NVALS);
    int nLvls = ljdbopen_(&unit, jg_name_c, varName, &varSize, &nVars);

    if (nLvls < 0) {
	char msg[1024];

        if (ljdberror_(&unit, &msg)) 
	    error += "\"Could not open " + jg_name + ": " + msg + "\"";
	else 
	    error += "\"JGOFS/DDS - could not open object: " +  jg_name + "\"";

	return false;
    }
    
    for (int v = 0; v < nVars; ++v) 
	varLevel[v] = ljdblevel_(&unit, &v);

    // When setting the dataset name, use only the part before the
    // constraint. 
    if ((pos = jg_name.find("(")) != jg_name.npos) {
	jg_name = jg_name.substr(0,pos);
    }
    dds.set_dataset_name(name_path(jg_name));

    JGSequence *seq;
    if ((seq = CreateSequence(0, nLvls))) 
	dds.add_var(seq);

    return true;
}
